
-- return the window size
function getWindowSize()
	return 595, 425
end

-- return the main font
-- the return is a list of font filename and size options
-- the first one found will be used
function getFont()
	if isMac then
		return { { name="GillSans.ttc", size=14 }, { name="GillSans.dfont", size=14 }, { name="Monaco.dfont", size=12 } }
	else
		return { { name="tahoma.ttf", size=14 }, { name="arial.ttf", size=12 } }
	end
end
-- return the label font
function getLabelFont()
	if isMac then
		return { { name="GillSans.ttc", size=11 }, { name="GillSans.dfont", size=11 }, { name="Monaco.dfont", size=9 } }
	else
		return { { name="tahoma.ttf", size=11 }, { name="arial.ttf", size=9 } }
	end
end
-- return the text field font
function getTextFieldFont()
	if isMac then
		return { { name="GillSans.ttc", size=12 }, { name="GillSans.dfont", size=12 }, { name="Monaco.dfont", size=10 } }
	else
		return { { name="tahoma.ttf", size=12 }, { name="arial.ttf", size=10 } }
	end
end

-- create and lay out all the controls
function createUI()

	-- load textures
	ripplesTexture = texture( "ripple.png" )

	-- constants
	local kKnobSize = 20
	local kKnobSpaceH = 20*1.1
	local kKnobSpaceV = 20*1.1
	local kTextFieldH = 12
	local pad = 10.0
	local ww, wh = getWindowSize()
	local kSliderWidth = 12
	local kSliderHeight = 90
	local kEvlSelSliderHeight = 15
	local kWfButtonW = 35
	local kWfButtonH = 25
	local kWfButtonSpace = kWfButtonH*1.1

	-- make renderers
	theKnobRenderer = knobRenderer()
	theRipples = ripples( ripplesTexture )
	theVortex = vortex( 0.5*ww, 0.5*wh, ww, wh )

	-- set location of the banner
	setBannerPosition( wh - 20 )

	-- set outline colours for groups
	setGroupActiveColour( 0.8, 0.75, 0.7, 0.8 )
	setGroupInactiveColour( 0.5, 0.5, 0.7, 0.5 )
	
	local function groupedKnob( group, param, label, x, y, size, textH, textW )
		textW = textW or size
		local knobx = x + 0.5 * ( textW - size )
		group:add( knob( param, knobx, y, size ) )
--		group:add( knob( param, "", knobx, y+textH+1, size ) )
--		group:add( textField( param, label, x, y, textW, textH ) )
	end

	-- make groups and knobs
	--

	local x = 0
	local y = 0

	modGroup = group( "Modulation" )
	groupedKnob( modGroup, "Modulation On/Off",		"On/Off",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Wet/Dry",	"Wet/Dry",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Delay",		"Delay",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Sweep",		"Sweep",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Rate",		"Rate",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Feedback",	"Feedback",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Spread",		"Spread",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Voices",		"Voices",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( modGroup, "Modulation Shape",		"Shape",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH

	x = 0
	y = 0
	velocityGroup = group( "Velocity" )
	groupedKnob( velocityGroup, "Velocity Env Amount",			"Env",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( velocityGroup, "Velocity Filter Amount",		"Filter",		x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( velocityGroup, "Velocity Filter Env Amount",	"Filter Env",	x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( velocityGroup, "Velocity PW Amount",			"PW",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( velocityGroup, "Velocity PW Env Amount",		"PW Env",		x, y, kKnobSize, kTextFieldH )

	x = 0
	y = 3*kKnobSpaceV
	miscGroup = group( "Misc" )
	groupedKnob( miscGroup, "Volume",				"Volume",			x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( miscGroup, "Pitch Wheel Range",	"Wheel",			x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( miscGroup, "Coarse Tune",			"Coarse",			x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( miscGroup, "Fine Tune",			"Fine",				x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV

	x = 0
	y = 0
	pwGroup = group( "PulseWidth" )
	groupedKnob( pwGroup, "PW Env Attack",			"Attack",			0*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( pwGroup, "PW Env Decay",			"Decay",			1*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( pwGroup, "PW Env Sustain",			"Sustain",			2*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( pwGroup, "PW Env Release",			"Release",			3*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( pwGroup, "Pulse Width",			"PW",				x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( pwGroup, "PW LFO Speed",			"LFO Spd",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( pwGroup, "PW LFO Amount",			"LFO Amt",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( pwGroup, "PW Env Amount",			"Env",				x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	pwGroup:add( adsrEnvDisplay( "pw", 0, 2*kKnobSpaceH, 4*kKnobSpaceV, 2*kKnobSize ) )

	x = 0
	y = 0
	filterModGroup = group( "Filter Mod" )
	groupedKnob( filterModGroup, "Filter Env Attack",			"Attack",			0*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( filterModGroup, "Filter Env Decay",			"Decay",			1*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( filterModGroup, "Filter Env Sustain",			"Sustain",			2*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( filterModGroup, "Filter Env Release",			"Release",			3*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( filterModGroup, "Filter LFO Speed",			"LFO Spd",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH	
	groupedKnob( filterModGroup, "Filter LFO Amount",			"LFO Amt",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( filterModGroup, "Filter LFO Type Amount",		"LFO Typ",			x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	groupedKnob( filterModGroup, "Filter Env Amount",			"Env",				x, y, kKnobSize, kTextFieldH )
	x = x + kKnobSpaceH
	filterModGroup:add( adsrEnvDisplay( "filter", 0, 2*kKnobSpaceH, 4*kKnobSpaceV, 2*kKnobSize ) )

	x = 0
	y = 0
	filterGroup = group( "Filter" )
	y = 3*kKnobSpaceV
	groupedKnob( filterGroup, "Filter Cutoff",			"Cutoff",		x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( filterGroup, "Filter Q",				"Q",			x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( filterGroup, "Filter Type",			"Type",			x, y, kKnobSize, kTextFieldH )
	y = y - kKnobSpaceV
	groupedKnob( filterGroup, "Filter KB Track",		"KB Trk",		x, y, kKnobSize, kTextFieldH )

	local function lfoSpeedKnobs( g, n )
		local x = (n-1) * kKnobSpaceH
		groupedKnob( g, "LFO " .. n .. " KB Track",			"KB Trk",		x, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "LFO " .. n .. " Speed",			"Speed",		x, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "LFO " .. n .. " Multiplier",		"Mult",			x, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
	end

	lfoSpeedsGroup = group( "LFO Speeds" )
	lfoSpeedKnobs( lfoSpeedsGroup, 1 )
	lfoSpeedKnobs( lfoSpeedsGroup, 2 )
	lfoSpeedKnobs( lfoSpeedsGroup, 3 )
	lfoSpeedKnobs( lfoSpeedsGroup, 4 )

	local function lfoEffectKnobs( g, n )
		local x = (n-1) * kKnobSpaceH
		groupedKnob( g, "LFO " .. n .. " Amount",			"Amount",		x, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "LFO " .. n .. " Offset",			"Offset",		x, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	end

	lfoEffectsGroup = group( "LFO Effects" )
	lfoEffectKnobs( lfoEffectsGroup, 1 )
	lfoEffectKnobs( lfoEffectsGroup, 2 )
	lfoEffectKnobs( lfoEffectsGroup, 3 )
	lfoEffectKnobs( lfoEffectsGroup, 4 )

	local function envGroup( n )
		local g = group( "Env " .. n )
		groupedKnob( g, "Env " .. n .. " Time A1",			"T A1",			0*kKnobSpaceH, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Time A2",			"T A2",			1*kKnobSpaceH, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Time A3",			"T A3",			2*kKnobSpaceH, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Time R1",			"T R1",			3*kKnobSpaceH, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Time R2",			"T R2",			4*kKnobSpaceH, 2*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Shape A1",			"Shp A1",		0*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Shape A2",			"Shp A2",		1*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Shape A3",			"Shp A3",		2*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Shape R1",			"Shp R1",		3*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Shape R2",			"Shp R2",		4*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Level A1",			"Lvl A1",		0.5*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Level A2",			"Lvl A2",		1.5*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Sustain",			"Sust",			2.5*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
		groupedKnob( g, "Env " .. n .. " Level R1",			"Lvl R1",		3.5*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
		g:add( envDisplay( n, 0, 3*kKnobSpaceH, 5*kKnobSpaceV, kKnobSize ) )
		return g
	end
	
	envGroup1 = envGroup( 1 )
	envGroup2 = envGroup( 2 )
	envGroup3 = envGroup( 3 )
	
	amplitudeGroup = group( "Amplitude" )
	groupedKnob( amplitudeGroup, "Attack",			"Attack",			0*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Decay",			"Decay",			1*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Sustain",			"Sustain",			2*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Release",			"Release",			3*kKnobSpaceH, 1*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Attack Shape",	"A Shp",			0*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Decay Shape",		"D Shp",			1*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
	groupedKnob( amplitudeGroup, "Release Shape",	"R Shp",			3*kKnobSpaceH, 0*kKnobSpaceV, kKnobSize, kTextFieldH )
	amplitudeGroup:add( adsrEnvDisplay( "amplitude", 0, 2*kKnobSpaceH, 4*kKnobSpaceV, 2*kKnobSize ) )

	levelsGroup = group( "Levels" )
	levelsGroup:add( levelsSliders( 0, 0, 16*kSliderWidth, kSliderHeight ) )

	envSelGroup = group( "Env Select" )
	envSelGroup:add( envSelSliders( 0, 0, 16*kSliderWidth, kEvlSelSliderHeight ) )

--	oscGroup = group( "OSC" )
--	groupedKnob( oscGroup, "OSC Port Offset",	"Port",			0, 0, kKnobSize, kTextFieldH )

	-- positions!

	local pad = 10

	local xbase = 20
	local ybase = 20
	
	x = xbase
	y = ybase

	levelsGroup:setPosition( x, y )
	y = y + levelsGroup:height() + pad

	envSelGroup:setPosition( x, y )
	y = y + envSelGroup:height() + pad

	local ybase2 = y
	
	x = xbase + levelsGroup:width() + pad
	y = ybase

	modGroup:setPosition( x, y )

	local wfx = x
	local wfy = y + modGroup:height()

	x = xbase
	y = ybase2

	amplitudeGroup:setPosition( x, y )
	x = x + amplitudeGroup:width() + pad
	
	local ybase3 = y + amplitudeGroup:height() + pad

	pwGroup:setPosition( x, y )
	x = x + pwGroup:width() + pad

	filterModGroup:setPosition( x, y )
	x = x + filterModGroup:width() + pad

	filterGroup:setPosition( x, y )
	x = x + filterGroup:width() + pad

	x = xbase + 3 * envGroup1:width() + 3 * pad
	velocityGroup:setPosition( x, y )
	y = y + velocityGroup:height() + pad

	lfoEffectsGroup:setPosition( x, y )
	y = y + lfoEffectsGroup:height() + pad

	lfoSpeedsGroup:setPosition( x, y )
	x = x + lfoSpeedsGroup:width() + pad

	miscGroup:setPosition( x, y )

--	y = y - pad - oscGroup:height()
--	oscGroup:setPosition( x, y )

	x = xbase
	y = ybase3

	envGroup1:setPosition( x, y )
	x = x + envGroup1:width() + pad

	envGroup2:setPosition( x, y )
	x = x + envGroup2:width() + pad

	envGroup3:setPosition( x, y )
	x = x + envGroup3:width() + pad

	-- add directly positioned objects
	--

	local h = ybase2 - 2*pad - wfy
	local kWaveformW = 256
	add( waveformDisplay( wfx, wfy, kWaveformW, h ) )

	x = wfx + kWaveformW + pad
	y = ybase - 10

	add( button( "Prefs", "Prefs", x, y, kWfButtonW, kWfButtonH ) )
	y = y + 40

	add( luaButton( setSqr, "Sqr", "Set square waveform", "push", x, y, kWfButtonW, kWfButtonH ) )
	y = y + kWfButtonSpace

	add( luaButton( setTri, "Tri", "Set triangle waveform", "push", x, y, kWfButtonW, kWfButtonH ) )
	y = y + kWfButtonSpace

	add( luaButton( setSaw, "Saw", "Set saw waveform", "push", x, y, kWfButtonW, kWfButtonH ) )
	y = y + kWfButtonSpace

	add( luaButton( setSin, "Sin", "Set sine waveform", "push", x, y, kWfButtonW, kWfButtonH ) )
	y = y + kWfButtonSpace
	
	-- get parameter IDs

	paramID_Level1 = getParameterID( "Level 1" )

end		-- of createUI()

function setSqr()
	for i=0,15,2 do
		setParameter( paramID_Level1 + i, 1.0/(1+i) )
	end
	for i=1,15,2 do
		setParameter( paramID_Level1 + i, 0.0 )
	end
end

function setTri()
	local s = 1
	for i=0,15,2 do
		setParameter( paramID_Level1 + i, s/((1+i)*(1+i)) )
		s = -s
	end
	for i=1,15,2 do
		setParameter( paramID_Level1 + i, 0.0 )
	end
end

function setSaw()
	for i=0,15,1 do
		setParameter( paramID_Level1 + i, 1.0/(1+i) )
	end
end

function setSin()
	setParameter( paramID_Level1 + 0, 1.0 )
	for i=1,15,1 do
		setParameter( paramID_Level1 + i, 0.0 )
	end
end
